@@ -1,19 +1,19 @@
 #![allow(unused_parens)]
 extern crate bytes;
-extern crate chrono;
 extern crate rss;
 extern crate thiserror;
 extern crate urlencoding;
 
-use std::collections::BTreeMap;
 use std::convert::TryFrom;
 
 use bytes::Buf;
 use bytes::Bytes;
+use itertools::Itertools;
 use rss::Channel;
 use rss::Item;
 use smartcow::SmartCow;
 use smartstring::alias::String;
+use tracing::instrument;
 
 mod error;
 pub use error::Error;
@@ -28,6 +28,7 @@ pub struct Client {
 }
 
 impl Client {
+	#[instrument(err, level = "info", skip(base_url, apikey), fields(base_url = %base_url.to_string(), apikey = %apikey.to_string()))]
 	pub fn new(base_url: impl ToString, apikey: impl ToString) -> Result<Self, reqwest::Error> {
 		let this = Self{
 			http: reqwest::Client::builder()
@@ -40,53 +41,60 @@ impl Client {
 		Ok(this)
 	}
 
-	async fn get(&self, t: &str, mut qparams: BTreeMap<&str, SmartCow<'_>>) -> Result<Bytes, reqwest::Error> {
-		qparams.insert("apikey", SmartCow::Borrowed(&self.apikey));
-		let url = format!("{}?t={}&{}", self.base_url, &t, qparams.into_iter()
-			.map(|(k, v)| format!("{}={}", urlencoding::encode(k), urlencoding::encode(&v)).into())
-			.collect::<Vec<String>>()
+	#[instrument(err, level = "debug", skip(self))]
+	async fn get(&self, t: &str, qparams: Vec<(&str, SmartCow<'_>)>) -> Result<Bytes, reqwest::Error> {
+		let url = format!("{}?t={}&apikey={}&{}", self.base_url, &t, self.apikey, qparams.into_iter()
+			.map(|(k, v)| {
+				let mut s = String::from(urlencoding::encode(k));
+				s.push('=');
+				s.push_str(&urlencoding::encode(&v));
+				s
+			})
 			.join("&")
 		);
-		Ok(self.http.get(&url).send().await?.error_for_status()?.bytes().await?)
+		self.http.get(&url).send().await?.error_for_status()?.bytes().await
 	}
 
-	async fn get_items(&self, t: &str, qparams: BTreeMap<&str, SmartCow<'_>>) -> Result<Vec<Item>, Error> {
-		let bytes = self.get(t, qparams).await?;
-		let channel = tokio::task::spawn_blocking(move || Channel::read_from(bytes.chunk())).await??;
+	#[instrument(err, level = "debug", skip(self))]
+	async fn get_items(&self, t: &str, qparams: Vec<(&str, SmartCow<'_>)>) -> Result<Vec<Item>, Error> {
+		let bytes = self.get(t, qparams).await?.reader();
+		let channel = Channel::read_from(bytes)?;
 		Ok(channel.into_items())
 	}
 
+	#[instrument(err, level = "info", skip(self))]
 	pub async fn tvsearch(&self, q: Option<&str>, rid: Option<u32>, tvdbid: Option<u32>, tvmazeid: Option<u32>, season: Option<u16>, ep: Option<u16>) -> Result<Vec<Result<Torrent, Error>>, Error> {
-		let mut qparams = BTreeMap::<&str, SmartCow>::new();
+		let mut qparams = Vec::new();
 		if let Some(v) = q {
-			qparams.insert("q", SmartCow::Borrowed(v));
+			qparams.push(("q", SmartCow::Borrowed(v)));
 		}
 		if let Some(v) = rid {
-			qparams.insert("rid", SmartCow::Owned(String::from(v.to_string())));
+			qparams.push(("rid", SmartCow::Owned(String::from(v.to_string()))));
 		}
 		if let Some(v) = tvdbid {
-			qparams.insert("tvdbid", SmartCow::Owned(String::from(v.to_string())));
+			qparams.push(("tvdbid", SmartCow::Owned(String::from(v.to_string()))))
 		}
 		if let Some(v) = tvmazeid {
-			qparams.insert("tvmazeid", SmartCow::Owned(String::from(v.to_string())));
+			qparams.push(("tvmazeid", SmartCow::Owned(String::from(v.to_string()))))
 		}
 		if let Some(v) = season {
-			qparams.insert("season", SmartCow::Owned(String::from(v.to_string())));
+			qparams.push(("season", SmartCow::Owned(String::from(v.to_string()))))
 		}
 		if let Some(v) = ep {
-			qparams.insert("ep", SmartCow::Owned(String::from(v.to_string())));
+			qparams.push(("ep", SmartCow::Owned(String::from(v.to_string()))))
 		}
 		let items = self.get_items("tvsearch", qparams).await?;
 		Ok(items.into_iter().map(Torrent::try_from).collect::<Vec<_>>())
 	}
 
+	#[instrument(err, level = "info", skip(self))]
 	pub async fn moviesearch(&self, q: Option<&str>, imdbid: Option<u64>) -> Result<Vec<Result<Torrent, Error>>, Error> {
-		let mut qparams = BTreeMap::<&str, SmartCow>::new();
+		let mut qparams = Vec::new();
 		if let Some(v) = q {
-			qparams.insert("q", SmartCow::Borrowed(v));
+			qparams.push(("q", SmartCow::Borrowed(v)))
 		}
 		if let Some(v) = imdbid {
-			qparams.insert("imdbid", SmartCow::Owned(String::from(v.to_string())));
+			qparams.push(("imdbid", SmartCow::Owned(String::from(v.to_string()))))
 		}
 		let items = self.get_items("movies", qparams).await?;
 		Ok(items.into_iter().map(Torrent::try_from).collect::<Vec<_>>())
